/**
 * @file
 * Entity Reference Tree JavaScript file.
 */

// Codes run both on normal page loads and when data is loaded by AJAX (or BigPipe!)
// @See https://www.drupal.org/docs/drupal-apis/javascript-api/javascript-api-overview
(function ($, Drupal, once) {
  /**
   * @namespace
   */
  Drupal.fullcalendar_block = {};

  Drupal.fullcalendar_block.instances = {};

  Drupal.behaviors.buildCalendarBlock = {
    attach: function (context, settings) {
      const dialogWidth = [400];
      const dialogType = ['modal'];
      const dialogOpen = [1];

      // Redirect to the URL
      function gotoURL(url) {
        const eventURL = new URL(url, location.origin);
        if (eventURL.origin !== "null") {
          window.open(url, '_blank').focus();
        }
      }

      // Event click callback
      function eventClick(info) {
        info.jsEvent.preventDefault();
        if (!info.event.url) {
          return;
        }

        const blockIndex = this.el.getAttribute('data-calendar-block-index');
        if (dialogOpen[blockIndex] === 1) {
          // Open in a dialog.
          const ajaxSettings = {
            url: info.event.url,
            dialogType: dialogType[blockIndex],
            dialog: {width: dialogWidth[blockIndex]}
          };
          const openDialog = Drupal.ajax(ajaxSettings).execute();

          openDialog.done(function() {
            // The modal dialog open successfully.
          }).fail(function() {
            // The Ajax modal couldn't open.
            // Open in a new tab instead.
            gotoURL(info.event.url);
          });
        }
        else {
          gotoURL(info.event.url);
        }
      }

      // Function to build calendar objects.
      function buildCalendars(context, settings) {
        $('.fullcalendar-block', context)
          .each(function() {
            const blockIndex = this.getAttribute('data-calendar-block-index');
            this.setAttribute('data-calendar-block-initialized', true);
            const blockSettings = settings.fullCalendarBlock && settings.fullCalendarBlock[blockIndex];
            if (!blockSettings) {
              console.warn('Could not fetch fullcalendar_block settings', blockIndex, this);
              return;
            }
            const calendarOptions = typeof blockSettings.calendar_options === 'string' ? JSON.parse(blockSettings.calendar_options) : $.extend(true, {}, blockSettings.calendar_options);

            if (typeof calendarOptions.events === 'string' && Drupal.url.isLocal(calendarOptions.events)) {
              // Check if there are any GET parameters to send to the internal
              // Drupal path. This should provide better integration with Views exposed filters.
              let queryString = window.location.search || '';

              if (queryString !== '') {
                // Remove the question mark and any Drupal path component,
                // existing fullcalendar search filters if any.
                const startParam = calendarOptions.startParam || 'start';
                let initialDate;
                const specialParams = [
                  // Internal Drupal paths.
                  'q', 'render',
                  // Remove parameters that will be sent by fullcalendar.
                  startParam,
                  calendarOptions.endParam || 'end',
                  calendarOptions.timeZoneParam || 'timeZone',
                ];

                if (typeof URLSearchParams !== 'undefined') {
                  const query = new URLSearchParams(queryString);
                  specialParams.forEach(function(specialParam) {
                    if (specialParam === startParam) {
                      initialDate = query.get(startParam);
                    }
                    query.delete(specialParam);
                  });
                  queryString = query.toString();
                }
                else {
                  // Fallback to strip out query strings.
                  // https://stackoverflow.com/a/1634841
                  const params = queryString.slice(1).split(/[&;]/g);
                  // Reverse iteration as may be destructive.
                  specialParams.forEach(function(specialParam) {
                    const prefix = encodeURIComponent(specialParam);
                    let i;
                    for (i = params.length; i-- > 0;) {
                      // Idiom for string.startsWith.
                      if (params[i] === prefix || (params[i].lastIndexOf(prefix + '=', 0) !== -1)) {
                        initialDate = params[i].split('=')[1];
                        if (initialDate) {
                          initialDate = decodeURIComponent(initialDate);
                        }
                        params.splice(i, 1);
                      }
                    }
                  });
                  queryString = params.join('&');
                }

                // Provide a default initial date if it's specified in the query string.
                if (!calendarOptions.initialDate && initialDate) {
                  const date = new Date(initialDate);
                  if (date instanceof Date && !isNaN(date)) {
                    // Valid date string.
                    calendarOptions.initialDate = initialDate;
                  }
                }

                if (queryString !== '') {
                  // If there is a '?' in events URL, & should be used to add
                  // parameters.
                  queryString = (/\?/.test(calendarOptions.events) ? '&' : '?') + queryString;
                  // Add the current query string to the query.
                  // Useful for taking in exposed filters.
                  calendarOptions.events = calendarOptions.events + queryString;
                }
              }
            }

            dialogOpen[blockIndex] = parseInt(blockSettings.dialog_open);
            dialogWidth[blockIndex] = parseInt(blockSettings.dialog_width);
            dialogType[blockIndex] = 'modal';

            // Bind the event click handler.
            calendarOptions.eventClick = eventClick;

            const calendar = new FullCalendar.Calendar(this, calendarOptions);
            calendar.render();
            // Store a reference to the calendar block instance.
            Drupal.fullcalendar_block.instances[blockIndex] = {
              element: this,
              index: blockIndex,
              calendar: calendar,
              calendarOptions: calendarOptions,
              blockSettings,
            };
            // Trigger an event to let other modules know when a calendar
            // has been built.
            $(context).trigger('fullcalendar_block.build', [Drupal.fullcalendar_block.instances[blockIndex]]);
          });
      }

      once('buildCalendarBlock', 'html', context).forEach( function () {
        // Build all calendars.
        buildCalendars(context, settings);
      })
    },
    detach: function (context, settings, trigger) {
      if (trigger === 'unload') {
        $('.fullcalendar-block[data-calendar-block-initialized]', context)
          .each(function() {
            // Unload the calendar block instance.
            const blockIndex = this.getAttribute('data-calendar-block-index');
            if (Drupal.fullcalendar_block.instances[blockIndex]) {
              if (Drupal.fullcalendar_block.instances[blockIndex].calendar) {
                Drupal.fullcalendar_block.instances[blockIndex].calendar.destroy();
              }
              delete Drupal.fullcalendar_block.instances[blockIndex];
              delete settings.fullcalendar_block[blockIndex];
            }
          });
      }
    }
  }
})(jQuery, Drupal, once);
