<?php

namespace Drupal\fullcalendar_block\Plugin\Block;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Link;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Url;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a FullCalendar Block.
 *
 * @Block(
 *   id = "fullcalendar_block",
 *   admin_label = @Translation("FullCalendar block"),
 *   category = @Translation("Calendar"),
 * )
 */
class FullCalendarBlock extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $instance = new static($configuration, $plugin_id, $plugin_definition);
    $instance->time = $container->get('datetime.time');

    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'event_source' => '',
      'initial_view' => 'dayGridMonth',
      'header_start' => 'prev,next today',
      'header_center' => 'title',
      'header_end' => 'timeGridWeek,timeGridDay',
      'open_dialog' => 1,
      'dialog_width' => 800,
      'advanced' => '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    // Our JavaScript needs to have some means to find the HTML belonging to
    // this view. Borrowing from views' "dom_id".
    //
    // In order to unequivocally match a block with its HTML, because multiple
    // calendar blocks may appear several times on the page.
    // So  we set up a hash with the current time, plugin_id, to issue a
    // "unique" identifier for each view. This identifier is written to both
    // drupalSettings and the fullcalendar_block DIV.
    $block_index = hash('sha256', $this->getPluginId() . $this->time->getRequestTime() . mt_rand());

    $config = $this->getConfiguration();
    $event_url = $config['event_source'];
    $initial_view = $config['initial_view'];
    $header_start = $config['header_start'];
    $header_center = $config['header_center'];
    $header_end = $config['header_end'];
    $dialog_open = $config['open_dialog'];
    $dialog_width = $config['dialog_width'];
    $advanced_settings = $config['advanced'];

    // Fullcalendar options.
    $calendar_options = [
      'initialView' => $initial_view,
      'events' => $this->resolveEventUrl($event_url),
      'headerToolbar' => [
        'start' => $header_start,
        'center' => $header_center,
        'end' => $header_end,
      ],
    ];

    if (!empty($advanced_settings)) {
      $calendar_options = array_merge($calendar_options, (array) json_decode($advanced_settings, TRUE));
    }

    $block_content = [
      '#theme' => 'fullcalendar_block',
      '#block_index' => $block_index,
      '#attached' => [
        'library' => [
          'fullcalendar_block/fullcalendar',
        ],
      ],
    ];

    $block_content['#attached']['drupalSettings']['fullCalendarBlock'][$block_index] = [
      'calendar_options' => $calendar_options,
      'dialog_open' => $dialog_open,
      'dialog_width' => $dialog_width,
    ];

    return $block_content;
  }

  /**
   * {@inheritdoc}
   */
  public function blockForm($form, FormStateInterface $form_state) {
    $form = parent::blockForm($form, $form_state);

    $config = $this->getConfiguration();

    $form['event_source'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Event source URL'),
      '#description' => $this->t('The URL where the calendar events data feeds'),
      '#default_value' => $config['event_source'],
      '#required' => TRUE,
    ];

    $form['initial_view'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Initial View'),
      '#description' => $this->t('The initial view of the calendar'),
      '#default_value' => $config['initial_view'],
    ];

    // Header toolbar settings.
    $form['header_toolbar'] = [
      '#type' => 'details',
      '#title' => $this->t('Header Toolbar'),
      '#description' => $this->t('Header toolbar of the calendar. <br/>See %doc for available options.', [
        '%doc' => Link::fromTextAndUrl($this->t('the help document'), Url::fromUri('https://fullcalendar.io/docs/headerToolbar', ['attributes' => ['target' => '_blank']]))->toString(),
      ]),
    ];

    $form['header_toolbar']['header_start'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Start of the header toolbar'),
      '#description' => $this->t('Start area will normally be on the left. if RTL, will be on the right'),
      '#default_value' => $config['header_start'],
    ];

    $form['header_toolbar']['header_center'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Center of the header toolbar'),
      '#description' => $this->t('The default value is title if leave this empty'),
      '#default_value' => $config['header_center'],
    ];

    $form['header_toolbar']['header_end'] = [
      '#type' => 'textfield',
      '#title' => $this->t('End of the header toolbar'),
      '#description' => $this->t('The default value is Week and Day view if leave this empty'),
      '#default_value' => $config['header_end'],
    ];

    // Click event settings.
    $form['click_event'] = [
      '#type' => 'details',
      '#title' => $this->t('Click event settings.'),
    ];

    $form['click_event']['open_dialog'] = [
      '#type' => 'radios',
      '#title' => $this->t('Click on an event'),
      '#options' => [
        0 => $this->t('Open in a new tab'),
        1 => $this->t('Open in a dialog'),
      ],
      '#default_value' => $config['open_dialog'],
      '#attributes' => [
        // Define static data condition attribute so we can easier select it.
        'data-condition' => 'field-open-dialog',
      ],
    ];

    $form['click_event']['dialog_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Dialog width'),
      '#default_value' => $config['dialog_width'],
      '#size' => '5',
      '#states' => [
        // Show this textfield only if 'open in a dialog' is selected above.
        'visible' => [
          ':input[data-condition="field-open-dialog"]' => ['value' => 1],
        ],
      ],
    ];

    // Advanced settings.
    $form['advanced'] = [
      '#type' => 'details',
      '#title' => $this->t('Advanced settings.'),
    ];

    $form['advanced']['addition'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Advanced settings in JSON format.'),
      '#default_value' => $config['advanced'],
      '#description' => $this->t('It must be in valid JSON format.<br/>See %doc for available options.', [
        '%doc' => Link::fromTextAndUrl($this->t('Fullcalendar Document'), Url::fromUri('https://fullcalendar.io/docs#toc', ['attributes' => ['target' => '_blank']]))->toString(),
      ]),
      '#element_validate' => [[$this, 'validateJson']],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function blockSubmit($form, FormStateInterface $form_state) {
    parent::blockSubmit($form, $form_state);
    $values = $form_state->getValues();
    $this->configuration['event_source'] = $values['event_source'];
    $this->configuration['initial_view'] = $values['initial_view'];
    $this->configuration['header_start'] = $values['header_toolbar']['header_start'];
    $this->configuration['header_center'] = $values['header_toolbar']['header_center'];
    $this->configuration['header_end'] = $values['header_toolbar']['header_end'];
    $this->configuration['open_dialog'] = (int) $values['click_event']['open_dialog'];
    $this->configuration['dialog_width'] = (int) $values['click_event']['dialog_width'];
    // Normalize the line endings.
    // https://www.drupal.org/node/3114725
    $this->configuration['advanced'] = trim(str_replace(["\r\n", "\r"], "\n", $values['advanced']['addition']));
  }

  /**
   * Callback to validate that the configuration is a valid JSON object.
   */
  public function validateJson(array $element, FormStateInterface $form_state) {
    $json = trim($element['#value']);

    if ($json) {
      $result = json_decode($json, TRUE);
      if (!is_array($result) || json_last_error() !== JSON_ERROR_NONE) {
        $form_state->setError($element, $this->t('%field must be a valid JSON object.', [
          '%field' => $element['#title'],
        ]));
      }
    }
  }

  /**
   * Resolves the current event URL relative to the current URL.
   *
   * Applying the appropriate URL prefixes to the endpoint as necessary.
   * Which is useful for multilingual Drupal instances.
   *
   * @param string $event_url
   *   The current URL.
   *
   * @return string
   *   The URL with the appropriate base prefix applied.
   */
  protected function resolveEventUrl($event_url) {
    if (!UrlHelper::isExternal($event_url)) {
      $event_url = ltrim($event_url, '/');
      if (!file_exists(DRUPAL_ROOT . '/' . $event_url)) {
        // Not a static file, resolve the relative path as normal.
        return Url::fromUri('internal:/' . $event_url)->toString();
      }
    }
    return $event_url;
  }

}
